##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Versioned Resources Implementation

$Id: resource.py 89589 2008-08-10 07:30:39Z srichter $
"""
__docformat__ = "reStructuredText"
import zope.component
from zope.publisher.interfaces import NotFound
from zope.app.publisher.browser import resource, resources
from zope.app.publisher.browser import directoryresource
from zope.app.publisher.browser import fileresource
from zope.app.publisher.browser import pagetemplateresource
from z3c.versionedresource import interfaces

class Resources(resources.Resources):

    def publishTraverse(self, request, name):
        '''See interface IBrowserPublisher'''
        try:
            return super(Resources, self).publishTraverse(request, name)
        except NotFound:
            pass

        vm = zope.component.queryUtility(interfaces.IVersionManager)
        if vm is None or vm.version != name:
            raise NotFound(self, name)

        res = resources.Resources(self.context, self.request)
        res.__name__ = vm.version
        return res


class VersionedResourceBase(object):

    def _createUrl(self, baseUrl, name):
        vm = zope.component.queryUtility(interfaces.IVersionManager)
        return '%s/@@/%s/%s' %(baseUrl, vm.version, name)

class Resource(VersionedResourceBase, resource.Resource):
    pass


class FileResource(VersionedResourceBase, fileresource.FileResource):
    # 10 years expiration date
    cacheTimeout = 10 * 365 * 24 * 3600

    def __repr__(self):
        return '<%s %r>' %(self.__class__.__name__, self.context.path)

class FileResourceFactory(fileresource.FileResourceFactory):
    resourceClass = FileResource

class ImageResourceFactory(fileresource.ImageResourceFactory):
    resourceClass = FileResource


class DirectoryResource(VersionedResourceBase,
                        directoryresource.DirectoryResource):

    resource_factories = {
        '.gif':  ImageResourceFactory,
        '.png':  ImageResourceFactory,
        '.jpg':  ImageResourceFactory,
        '.pt':   pagetemplateresource.PageTemplateResourceFactory,
        '.zpt':  pagetemplateresource.PageTemplateResourceFactory,
        }

    default_factory = FileResourceFactory
    directory_factory = None

    def __repr__(self):
        return '<%s %r>' %(self.__class__.__name__, self.context.path)

class DirectoryResourceFactory(directoryresource.DirectoryResourceFactory):
    factoryClass = DirectoryResource

DirectoryResource.directory_factory = DirectoryResourceFactory



##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Versioned Resources Interfaces

$Id: interfaces.py 89405 2008-08-05 16:58:50Z srichter $
"""
__docformat__ = "reStructuredText"
import zope.interface
import zope.schema

class IVersionManager(zope.interface.Interface):
    """A manager for resource versions."""

    version = zope.schema.ASCIILine(
        title=u'Version',
        description=u'The versoin of the resources.',
        required=True)

class IVersionedResource(zope.interface.Interface):
    """Versioned Resource"""



##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Versioned Resources Test Setup

$Id: test_doc.py 91567 2008-09-28 05:53:11Z srichter $
"""
__docformat__ = "reStructuredText"
import os
import unittest
import zope.interface
from zope.component import globalregistry
from zope.testing import doctest
from zope.app.testing import placelesssetup
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from z3c.versionedresource import interfaces, resource

class ITestLayer(IDefaultBrowserLayer):
    pass

class ITestLayer2(IDefaultBrowserLayer):
    pass

class ResourceFactory(object):

    def __init__(self, request):
        self.request = request

    def __repr__(self):
        return '<%s>' %self.__class__.__name__

def unregister(name):
    reg = globalregistry.globalSiteManager.adapters
    reg.unregister((IDefaultBrowserLayer,), interfaces.IVersionedResource, name)

def ls(dir):
    for name in sorted(os.listdir(dir)):
        path = os.path.join(dir, name)
        print '%s %s\t%i' %(
            'd' if os.path.isdir(path) else 'f',
            name,
            os.stat(path).st_size)

def test_suite():
    return unittest.TestSuite((
        doctest.DocFileSuite(
            '../README.txt',
            setUp=placelesssetup.setUp, tearDown=placelesssetup.tearDown,
            globs = {'ls': ls, 'unregister': unregister},
            optionflags=doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS),
        ))



##############################################################################
#
# Copyright (c) 2008 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Resource List

$Id: list.py 91567 2008-09-28 05:53:11Z srichter $
"""
__docformat__ = 'restructuredtext'
import gzip
import optparse
import os
import sys
import zope.component
import zope.interface
from zope.configuration import xmlconfig
from zope.publisher.browser import TestRequest
from z3c.versionedresource import interfaces, resource
from z3c.versionedresource.resource import DirectoryResource

EXCLUDED_NAMES = ('.svn',)

def removeExcludedNames(list):
    for name in EXCLUDED_NAMES:
        if name in list:
            list.remove(name)

def getResources(layerPaths, url='http://localhost/'):
    resources = ()
    for layerPath in layerPaths:
        # Get the layer interface
        moduleName, layerName = layerPath.rsplit('.', 1)
        module = __import__(moduleName, {}, {}, ['None'])
        layer = getattr(module, layerName)
        # Now we create a test request with that layer and our custom base URL.
        request = TestRequest(environ={'SERVER_URL': url})
        zope.interface.alsoProvides(request, layer)
        # Next we look up all the resources
        resources += tuple(
            zope.component.getAdapters(
                (request,), interfaces.IVersionedResource))
    return resources

def getResourceUrls(resources):
    paths = []
    for name, res in resources:
        # For file-based resources, just report their URL.
        if not isinstance(res, DirectoryResource):
            # Avoid duplicate paths
            if res() not in paths:
                paths.append(res())
        # For directory resources, we want to walk the tree.
        baseURL = res()
        path = res.context.path
        for root, dirs, files in os.walk(path):
            # Ignore unwanted names.
            removeExcludedNames(dirs)
            removeExcludedNames(files)
            # Produce a path for the resource
            relativePath = root.replace(path, '')
            for file in files:
                fullPath = baseURL + relativePath + '/' + file
                # Avoid duplicate paths
                if fullPath not in paths:
                    paths.append(fullPath)
    return paths

def storeResource(dir, name, resource, zip=False):
    outputPath = os.path.join(dir, name)
    # For directory resources, we create the directory and walk through the
    # children.
    if isinstance(resource, DirectoryResource):
        if not os.path.exists(outputPath):
            os.mkdir(outputPath)
        for name in [name for name in os.listdir(resource.context.path)
                     if name not in EXCLUDED_NAMES]:
            subResource = resource.get(name, None)
            if subResource is not None:
                storeResource(outputPath, name, subResource, zip)
    # For file-based resources, get the path, load it and store it at the new
    # location.
    else:
        inFile = open(resource.context.path, 'r')
        openFile = open
        if zip:
            openFile = gzip.open
        outFile = openFile(outputPath, 'w')
        outFile.write(inFile.read())
        inFile.close()
        outFile.close()

def produceResources(options):
    # Run the configuration
    context = xmlconfig.file(options.zcml)
    # Get resource list
    resources = getResources(options.layers, options.url)
    # If we only want to list the paths
    if options.listOnly:
        paths = getResourceUrls(resources)
        print '\n'.join(paths)
        return
    # Now we can produce the version directory with all resources in it
    version = zope.component.getUtility(interfaces.IVersionManager).version
    outputdir = os.path.join(options.dir, version)
    if not os.path.exists(outputdir):
        os.mkdir(outputdir)
    for name, resource in resources:
        storeResource(outputdir, name, resource, options.zip)
    print outputdir

###############################################################################
# Command-line UI

parser = optparse.OptionParser("%prog [options] ROOT-ZCML-FILE")

config = optparse.OptionGroup(
    parser, "Configuration", "Configuration of lookup and reporting parameters.")

config.add_option(
    '--layer', '-l', action="append", dest='layers',
    help="""The layer for which to lookup the resources.""")

config.add_option(
    '--zip', '-z', action="store_true", dest='zip', default=False,
    help="""When set, resources are stored in GZIP format.""")

config.add_option(
    '--output-dir', '-d', action="store", dest='dir',
    default=os.path.abspath(os.curdir),
    help="""The directory in which the resources will be stored.""")

parser.add_option_group(config)

list = optparse.OptionGroup(
    parser, "List", "Options for list only result.")

list.add_option(
    '--list-only', action="store_true", dest='listOnly', default=False,
    help="When specified causes no directory with the resources "
         "to be created.")

list.add_option(
    '--url', '-u', action="store", dest='url', default='http://localhost/',
    help="""The base URL that is used to produce the resource URLs.""")

parser.add_option_group(list)

def get_options(args=None):
    if args is None:
        args = sys.argv
    original_args = args
    options, positional = parser.parse_args(args)
    options.original_args = original_args

    if not positional or len(positional) < 1:
        parser.error("No target user and/or packages specified.")
    options.zcml = positional[0]

    return options

# Command-line UI
###############################################################################

def main(args=None):
    options = get_options(args)
    try:
        produceResources(options)
    except Exception, err:
        print err
        sys.exit(1)
    sys.exit(0)



##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Version Manager Implementation

$Id: version.py 89405 2008-08-05 16:58:50Z srichter $
"""
__docformat__ = "reStructuredText"

import zope.interface
from zope.schema.fieldproperty import FieldProperty
from z3c.versionedresource import interfaces


class VersionManager(object):
    zope.interface.implements(interfaces.IVersionManager)

    version = FieldProperty(interfaces.IVersionManager['version'])

    def __init__(self, version):
        self.version = version

    def __repr__(self):
        return '<%s %r>' %(self.__class__.__name__, self.version)



##############################################################################
#
# Copyright (c) 2008 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Resource Directives

$Id: zcml.py 89405 2008-08-05 16:58:50Z srichter $
"""
__docformat__ = 'restructuredtext'
import os
from zope.app.publisher.browser.resourcemeta import ResourceFactoryWrapper
from zope.app.publisher.browser.resourcemeta import allowed_names
from zope.app.publisher.browser.pagetemplateresource import \
     PageTemplateResourceFactory
from zope.component.zcml import handler
from zope.configuration.exceptions import ConfigurationError
from zope.interface import Interface
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.security.checker import CheckerPublic, NamesChecker

from z3c.versionedresource import interfaces
from z3c.versionedresource.resource import DirectoryResourceFactory
from z3c.versionedresource.resource import FileResourceFactory
from z3c.versionedresource.resource import ImageResourceFactory


def resource(_context, name, layer=IDefaultBrowserLayer,
             permission='zope.Public', factory=None,
             file=None, image=None, template=None):

    if permission == 'zope.Public':
        permission = CheckerPublic

    checker = NamesChecker(allowed_names, permission)

    if (factory and (file or image or template)) or \
       (file and (factory or image or template)) or \
       (image and (factory or file or template)) or \
       (template and (factory or file or image)):
        raise ConfigurationError(
            "Must use exactly one of factory or file or image or template"
            " attributes for resource directives"
            )

    if factory is not None:
        factory = ResourceFactoryWrapper(factory, checker, name)
    elif file:
        factory = FileResourceFactory(file, checker, name)
    elif image:
        factory = ImageResourceFactory(image, checker, name)
    else:
        factory = PageTemplateResourceFactory(template, checker, name)

    _context.action(
        discriminator = ('resource', name, IBrowserRequest, layer),
        callable = handler,
        args = ('registerAdapter', factory, (layer,),
                interfaces.IVersionedResource, name, _context.info),
        )


def resourceDirectory(_context, name, directory, layer=IDefaultBrowserLayer,
                      permission='zope.Public'):
    if permission == 'zope.Public':
        permission = CheckerPublic

    checker = NamesChecker(allowed_names + ('__getitem__', 'get'),
                           permission)

    if not os.path.isdir(directory):
        raise ConfigurationError(
            "Directory %s does not exist" % directory
            )

    factory = DirectoryResourceFactory(directory, checker, name)
    _context.action(
        discriminator = ('resource', name, IBrowserRequest, layer),
        callable = handler,
        args = ('registerAdapter', factory, (layer,),
                interfaces.IVersionedResource, name, _context.info),
        )
